using System;

namespace PhidgetServer
{
	/// <summary>
	/// PhidgetCommand ̊Tv̐łB
	/// </summary>
	public class Command
	{
		//\PbgʐM̋؂q
		public const string DELIMITER_ARGS = ",";
		public static string DELIMITER_LINE = "\r\n";

		public class States
		{
			public const string In = "In";
			public const string Out = "Out";
			public const string Attach = "Attach";
			public const string Detach = "Detach";
			public const string Threshold = "Threshold";
			//public const string Connect = "Connect";
			//public const string Disconnect = "Disconnect";
		}

		public class Types
		{
			public const string Weight = "Weight";
			public const string InterfaceKit = "InterfaceKit";
			public const string RFID = "RFID";
			public const string Accel = "Accel";
			public const string Servo = "Servo";
		}

		public class SubTypes
		{
			public const string Digital = "Digital";
			public const string Analog = "Analog";
		}
			
		public abstract class Phidget
		{
			protected string state = "";
			protected string type = "";
			
			public string State
			{
				get{return this.state;}
			}

			public string Type
			{
				get{return this.type;}
			}

			public static Phidget Parse(string str)
			{
				Phidget phidget = null;
				const int ARGS_LENGTH_MIN = 2;

				string [] args = str.Split(Command.DELIMITER_ARGS.ToCharArray());
					
				if(args.Length < ARGS_LENGTH_MIN)
					return null;

				switch(args[0]){
					case Command.States.Out:
					{
						switch(args[1])
						{
							case Command.Types.Servo:
								phidget = Command.Servo.Parse(args);
								break;
							case Command.Types.InterfaceKit:
								phidget = Command.InterfaceKit.Parse(args);
								break;
							default:
								break;
						}
						break;
					}
					case Command.States.Threshold:
					{
						switch(args[1])
						{
							case Command.Types.InterfaceKit:
								phidget = Command.InterfaceKit.Parse(args);
								break;
							case Command.Types.Accel:
								phidget = Command.Accel.Parse(args);
								break;
							case Command.Types.RFID:
								phidget = Command.RFID.Parse(args);
								break;
							case Command.Types.Weight:
								phidget = Command.Weight.Parse(args);
								break;
						}
						break;
					}
				}
				return phidget;
			}
		}

		public class InterfaceKit : Phidget
		{
			protected string subType = "";
			int port = 0;
			int data = 0;

			public const int PORT_MAX = 8;
			//public const int DATA_MAX = 1024;
			
			public InterfaceKit(string state, string subType, int port, int data)
			{
				this.state = state;
				this.type = Command.Types.InterfaceKit;
				this.subType = subType;
				this.port = port;
				this.data = data;
			}

			public InterfaceKit(string state, int port, int data)
			{
				this.state = state;
				this.type = Command.Types.InterfaceKit;
				
				switch(state)
				{
					case Command.States.Out:
						this.subType = Command.SubTypes.Digital;
						break;
					case Command.States.Threshold:
						this.subType = Command.SubTypes.Analog;
						break;
					case Command.States.In:
						this.subType = Command.SubTypes.Analog;
						break;
				}
				this.port = port;
				this.data = data;
			}

			public InterfaceKit(string state)
			{
				this.state = state;
				this.type = Command.Types.InterfaceKit;
			}

			public override string ToString()
			{
				string str = "";
				switch(state)
				{
					case Command.States.Attach:
					case Command.States.Detach:
						str = String.Format("{2}{0}{3}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type);
						break;
					case Command.States.In:
						str = String.Format("{2}{0}{3}{0}{4}{0}{5}{0}{6}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type,
							subType,
							port,
							data);
						break;
				}
				return str;
			}

			public static InterfaceKit Parse(string []args)
			{
				try
				{	
					InterfaceKit interfaceKit = 
						new InterfaceKit(args[0], int.Parse(args[2]), int.Parse(args[3]));
					return interfaceKit;
				}
				catch(Exception ex)
				{
					return null;
				}
			}

			public string SubType
			{
				get{return this.subType;}
			}

			public int Port
			{
				get{return port;}
			}

			public int Data
			{
				get{return data;}
			}

		}

		public class Weight : Phidget
		{
			double data = 0.0;
				
			public Weight(string state, double data)
			{
				this.state = state;	
				this.type = Command.Types.Weight;	
				this.data = data;
			}

			public Weight(string state)
			{
				this.state = state;	
				this.type = Command.Types.Weight;	
			}
			

			public static Weight Parse(string []args)
			{
				try
				{
					Weight weight = 
						new Weight(args[0], double.Parse(args[2]));
					return weight;
				}
				catch(Exception ex)
				{
					return null;
				}
			}

			public double Data
			{
				get{return this.data;}
			}

			public override string ToString()
			{
				string str = "";
				switch(state)
				{
					case Command.States.Attach:
					case Command.States.Detach:
						str = String.Format("{2}{0}{3}{1}", 
									DELIMITER_ARGS, 
									DELIMITER_LINE,
									state, 
									type);
						break;
					case Command.States.In:
						str = String.Format("{2}{0}{3}{0}{4:0.0}{1}", 
									DELIMITER_ARGS, 
									DELIMITER_LINE,
									state, 
									type, 
									data);
						break;
				}
				return str;
			}
		
		}

		public class RFID : Phidget
		{
			
			string data = "";

			public RFID(string state, string data)
			{
				this.state = state;
				this.type = Command.Types.RFID;
				this.data = data;
			}

			public RFID(string state)
			{
				this.state = state;
				this.type = Command.Types.RFID;
			}

			public static RFID Parse(string []args)
			{
				try
				{
					RFID rfid = 
						new RFID(args[0], args[2]);
					return rfid;
				}
				catch(Exception ex)
				{
					return null;
				}
			}

			public override string ToString()
			{
				string str = "";
				switch(state)
				{
					case Command.States.Attach:
					case Command.States.Detach:
						str = String.Format("{2}{0}{3}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type);
						break;
					case Command.States.In:
						str = String.Format("{2}{0}{3}{0}{4}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type,
							data);
						break;
				}
				return str;
			}

			public string Data
			{
				get{return data;}
			}
		}

		public class Accel : Phidget
		{
			int port = 0;		
			double data = 0.0;

			public Accel(string state, int port, double data)
			{
				this.state = state; 
				this.type = Command.Types.Accel;
				this.port = port;
				this.data = data;
			}

			public Accel(string state)
			{
				this.state = state;
				this.type = Command.Types.Accel;
			}

			public static Accel Parse(string []args)
			{
				try
				{
					Accel accel = 
						new Accel(args[0],int.Parse(args[2]),double.Parse(args[3]));
					return accel;
				}
				catch(Exception ex)
				{
					return null;
				}
			}

			public override string ToString()
			{
				string str = "";
				switch(state)
				{
					case Command.States.Attach:
					case Command.States.Detach:
						str = String.Format("{2}{0}{3}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type);
						break;
					case Command.States.In:
						str = String.Format("{2}{0}{3}{0}{4}{0}{5:0.00}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type,
							port,
							data);
						break;
				}
				return str;
			}

			public int Port
			{
				get{return port;}
			}

			public double Data
			{
				get{return data;}
			}
		}

		public class Servo : Phidget
		{
			int port = 0;
			int data = 0;

			public const int PORT_MAX = 4;
				
			public Servo(string state, int port, int data)
			{
				this.state = state;
				this.type = Command.Types.Servo;
				this.port = port;
				this.data = data;
			}

			public Servo(string state)
			{
				this.state = state;
				this.type = Command.Types.Servo;
			}

			public override string ToString()
			{
				string str = "";
				switch(state)
				{
					case Command.States.Attach:
					case Command.States.Detach:
						str = String.Format("{2}{0}{3}{1}", 
							DELIMITER_ARGS, 
							DELIMITER_LINE,
							state, 
							type);
						break;
				}
				return str;
			}

			public int Port
			{
				get{return port;}
			}

			public int Data
			{
				get{return data;}
			}

			public static Servo Parse(string [] args)
			{
				try
				{
					Servo servo = 
						new Servo(args[0], int.Parse(args[2]), int.Parse(args[3]));
					return servo;
				}
				catch
				{
					return null;
				}
			}

		}
	}
}
